package webapps.ajax1.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.annotation.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import com.google.gson.*;
import webapps.ajax1.grid.*;
import webapps.ajax1.model.*;

/**
 * Servlet que da soporte a grillas jtable server side
 * 
 * INPUT:
 *     parametro "name" nombre de la grilla a recuperar dentro del map de grillas
 *     previamente cargadas
 *     separadas por "," (coma)
 * OUTPUT: Devuelve arreglo asociativo para la creacion de objeto jtable:
 *   cuando hay todo Ok: { "Result":"OK","jtable":{<arreglo jtable>}}
 *   cuando hay error: { "Result":"ERROR","Message":"mensaje de error"}
 * 
 * @author G. Cherencio.
 * @version 1.0
 */
@WebServlet(name = "GridServlet", urlPatterns = {"/grid"})
public class GridServlet  extends HttpServlet {
    private static Map<String,SSTable> mapt = new HashMap<String,SSTable>();
    private static FBCon fb1 = FBCon.getFBCon(1);  // isft database
    private final static GsonBuilder gsonBuilder = new GsonBuilder();

    static {
        gsonBuilder.disableHtmlEscaping();  // evitar secuencia de escape en caracter apostrofe o comilla simple
        //try {
            //ResultSet rs = fb1.getResultSet("SELECT IDM,FECHA,CODIGOM FROM TBL_MESAEX WHERE 1 = 0");
            //SSTable t = new SSTable(rs);
            SSTable t = new SSTable();
            t.setName("mesaex");
            t.setField("idm:{title:Mesa #}");
            //t.getField("idm").setField(new SSOption("title","string",null,"Mesa #"));
            //SSField f = t.getField("fecha");
            //f.setField(new SSOption("title","string",null,"Fecha"));
            //f.setField(new SSOption("type","string",null,"date"));
            //f.setField(new SSOption("displayFormat","string",null,"dd/mm/yy"));
            t.setField("fecha:{title:Fecha,type:date,displayFormat:dd/mm/yy}");
            //t.getField("codigom").setField(new SSOption("title","string",null,"Materia"));
            t.setField("codigom:{title:Materia}");
            //fb1.close(rs);
            //t.addAction(new SSOption("listAction","string",null,"JsonServletDates"));
            t.setAction("listAction","JsonServletDates");
            // agrego grilla al conjunto de grillas previamente definidas
            mapt.put(t.getName(),t);
        //} catch(SQLException sqle) {
        //}
    }

    /**
     * Processes requests for both HTTP
     * <code>GET</code> and
     * <code>POST</code> methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setCharacterEncoding("UTF8");
        response.setContentType("application/json");
        String gridName = request.getParameter("name");
        Map<String,Object> mapv = new HashMap<String,Object>();
System.out.println("gridName="+gridName);
        PrintWriter out = response.getWriter();
        if ( gridName != null ) {
            SSTable tbl = mapt.get(gridName);
            if ( tbl != null ) {
                mapv.put("Result","OK");
System.out.println("jtable.generate="+tbl.generate()+"=");                
                mapv.put("jtable",tbl.generate());
                String json="{\"Result\":\"OK\",\"jtable\":"+tbl.generate()+"}";
System.out.println("json len="+json.length()+" pos=628-635"+json.substring(628,635)+" json="+json+"=");                
                out.print(json);
                out.close();
                return;
            } else {
                mapv.put("Result","ERROR");
                mapv.put("Message","Grilla["+gridName+"] no existe!");
            }
        } else {
            mapv.put("Result","ERROR");
            mapv.put("Message","Atributo [name] no indicado!");
        }
        // convertir json, retornar
        Gson gson = gsonBuilder.create();
        String json = gson.toJson(mapv);
        out.print(json);
System.out.println("json="+json+"=");
        out.close();
    }

    
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
